//  File: Thermistor.cpp
//
//  Use a voltage divider to indicate electrical resistance of a thermistor
//  Thermistor reading and T conversion are encapsulated in reusable objects
//
//  Based on code by Max Mayfield, max.mayfield@hotmail.com
//  posted on the Arduino playground
//  http://www.arduino.cc/playground/ComponentLib/Thermistor2
//  Modified and extended by G. Recktenwald, recktenwaldg@gmail.com
//  2010-06-07

#include "WProgram.h"
#include "Thermistor.h"

// --- constructor
Thermistor::Thermistor(int pin) {
  _pin = pin;              //  Analog input pin to read voltage divider output
  _Rfix = 9970;            //  Resistance of fixed resistor in the divider
  _a1 = 1.009249522e-03;   //  Coefficients in the Steinhart-Hart equation
  _a2 = 2.378405444e-04;
  _a3 = 2.019202697e-07;
}

// --- Set the value of the fixed resistance
void Thermistor::fixedResistance(float Rfix) {
  _Rfix = Rfix;
}

// --- Set values of coefficients in the Steinhart-Hart equation
void Thermistor::coefficients(float a1, float a2, float a3) {
  _a1 = a1;
  _a2 = a2;
  _a3 = a3;
}

// --- Read the voltage across the fixed resistance, and from it compute T
float Thermistor::getTemp() {

  int vin  = analogRead(_pin);     //  voltage corresponding to thermistor Rt
  float Rt;
  float logRt, T;

  Rt = _Rfix * ( ( 1023.0 / float(vin) ) - 1.0 );
  logRt = log(Rt);
  T = ( 1.0 / (_a1 + _a2*logRt + _a3*logRt*logRt*logRt ) ) - 273.15;

  return T;
}
